/**
 * Created by tingkl on 16/8/1.
 */
var $ = require('jquery');
var jss = require('./jss');
function CrossMove(selector, currentPoint, pointMap, direction, roadHandler) {
	this.$dom = $(selector);
	this.direction = direction; // down up left right
	this.pointMap = this.reformPoint(pointMap);
	var position = pointMap[currentPoint].coordinate;
	this.position = {x: position.x, y: position.y};
	this.listener = [];
	this.roadHandler = this.reformRoad(roadHandler);
	this.currentPoint = currentPoint;
	this.prePoint = 'start';
	this.weightX = 0;
	this.weightY = 0;
	this.nextRoad = '';
}
CrossMove.prototype.getPoint = function () {
	var x = this.position.x;
	var y = this.position.y;
	var point;
	var find = false;
	for (var pointName in this.pointMap) {
		point = this.pointMap[pointName].coordinate;
		if (x == point.x && y == point.y) {
			find = pointName;
			break;
		}
	}
	return find;
};
CrossMove.prototype.reformPoint = function (points) {
	var fromCss = points.fromCss;
	var point;
	var css;
	if (fromCss) {
		delete points.fromCss;
		for (var key in points) {
			point = points[key];
			if (!point.coordinate) {
				css = jss.getAll('.' + key);
				point.coordinate = {x: Number(css.left.replace('px', '')), y: Number(css.top.replace('px', ''))};
			}
		}
		//console.log(JSON.stringify(points));
	}
	return points;
};
CrossMove.prototype.reformRoad = function (roads) {
	var starEnd;
	for (var key in roads) {
		starEnd = key.split(' ');
		roads[key].startPoint = this.pointMap[starEnd[0]].coordinate;
		roads[key].endPoint = this.pointMap[starEnd[1]].coordinate;
	}
	return roads;
};
CrossMove.prototype.inRegion = function (pa, pb) {
	var x = this.position.x;
	var y = this.position.y;
	var x1 = (pa.x > pb.x) ? pb.x : pa.x;
	var x2 = pa.x + pb.x - x1;
	var y1 = (pa.y > pb.y) ? pb.y : pa.y;
	var y2 = pa.y + pb.y - y1;
	if (x >= x1 && x <= x2 && y >= y1 && y <= y2) {
		return true;
	}
	return false;
};
CrossMove.prototype.getRoad = function () {
	var handler;
	for (var key in this.roadHandler) {
		handler = this.roadHandler[key];
		if (this.inRegion(handler.startPoint, handler.endPoint)) {
			return key;
		}
	}
	return false;
};

CrossMove.prototype.setDirection = function (offsetX, offsetY) {
	this.direction = '';
	this.weightX = offsetX;
	this.weightY = offsetY;
	if (offsetX < 0) {
		this.direction = 'left';
	}
	else if (offsetX > 0) {
		this.direction = 'right';
	}
	if (offsetY < 0) {
		this.direction += 'up';
	}
	else if (offsetY > 0) {
		this.direction += 'down';
	}
};
CrossMove.prototype.start = function () {
	if (this.run) {
		return;
	}
	this.run = true;
	this.animate();
};

CrossMove.prototype.onPoint = function (callback) {
	this.listener.push(callback);
};
CrossMove.prototype.emitPoint = function (evt) {
	if (evt !== this.currentPoint) {
		this.prePoint = this.currentPoint;
		this.currentPoint = evt;
		for (var i = 0; i < this.listener.length; i++) {
			this.listener[i](evt);
		}
	}
};
CrossMove.prototype.nextPoint = function (endPoint) {
	if (this.currentPoint.length < endPoint.length) {
		this.nextRoad = this.currentPoint + ' ' + endPoint;
	}
	else if (this.currentPoint.length > endPoint.length) {
		this.nextRoad = endPoint + ' ' + this.currentPoint;
	}
	else {
		if (endPoint > this.currentPoint) {
			this.nextRoad = this.currentPoint + ' ' + endPoint;
		}
		else {
			this.nextRoad = endPoint + ' ' + this.currentPoint;
		}
	}

};
CrossMove.prototype.animate = function () {
	if (this.run) {
		var me = this;
		window.requestAnimationFrame(function () {
			var position = me.position;
			var road;
			if (me.nextRoad) {
				road = me.nextRoad;
				me.nextRoad = '';
			}
			else {
				road = me.getPoint() || me.getRoad();
			}
			//console.log(me.direction, road);
			if (road.indexOf(' ') < 0) {
				//point
				me.emitPoint(road);
				var targetHandler = me.pointMap[road];
				if (targetHandler.cross && !targetHandler.handler) {
					targetHandler.handler = (function (cross) {
						return function (prePoint, direction) {
							var cell;
							for (var i = 0; i < cross.length; i++) {
								cell = cross[i];
								if (cell.direction.indexOf('||')) {
									var ors = cell.direction.split('||');
									var match = false;
									for (var j = 0; j < ors.length; j++) {
										if (direction.indexOf(ors[j]) > -1) {
											match = true;
											break;
										}
									}
									if (match) {
										me.nextPoint(cell.point);
										me.direction = cell.turn || me.direction;
										break;
									}
								}
								else if (cell.direction.indexOf('&&')) {
									var ands = cell.direction.split('&&');
									var match = true;
									for (var j = 0; j < ands.length; j++) {
										if (direction.indexOf(ands[j]) < 0) {
											match = false;
											break;
										}
									}
									if (match) {
										me.nextPoint(cell.point);
										me.direction = cell.turn || me.direction;
										break;
									}
								}
								else if (direction.indexOf(cell.direction) > -1) {
									me.nextPoint(cell.point);
									me.direction = cell.turn || me.direction;
									break;
								}
							}
						}
					})(targetHandler.cross);
				}
				targetHandler.handler.call(me, me.prePoint, me.direction);
			}
			else {
				//road
				var targetHandler = me.roadHandler[road];
				//指定路径的情况
				if (targetHandler.paths) {
					if (!targetHandler._handler) {
						targetHandler._handler = (function (handler) {
							var paths = handler.paths;
							/*paths.push(handler.endPoint);
							 paths.unshift(handler.startPoint);*/
							var length = handler.paths.length; //0-9
							// 0 1 2 3 4 5
							// 6
							// 1-5  4-0
							var index = 1;
							var direction = '+';
							return function (startPoint, endPoint, road) {
								var position = paths[index];
								if (handler.handler) {
									handler.handler.call(me, startPoint, endPoint, road, position, direction);
								}
								else {
									me.position.x = position.x;
									me.position.y = position.y;
								}
								if (direction == '+') {
									if (index < length - 1) {
										index++;
									}
									else {
										direction = '-';
										index = length - 2;
									}
								}
								else {
									if (index > 0) {
										index--;
									}
									else {
										direction = '+';
										index = 1;
									}
								}
							}
						})(targetHandler);
					}
					targetHandler._handler.call(me, targetHandler.startPoint, targetHandler.endPoint, road);
				}
				else {
					if (!targetHandler.handler) {
						if (targetHandler.startPoint.x === targetHandler.endPoint.x) {
							//yMove
							targetHandler.handler = (function (startPoint, endPoint) {
								var y1 = (startPoint.y > endPoint.y) ? endPoint.y : startPoint.y;
								var y2 = startPoint.y + endPoint.y - y1;
								return function () {
									if (this.direction.indexOf('up') > -1) {
										if (this.position.y > y1) {
											this.position.y--;
										}
									} else if (this.direction.indexOf('down') > -1) {
										if (this.position.y < y2) {
											this.position.y++;
										}
									}
								}
							})(targetHandler.startPoint, targetHandler.endPoint);
						}
						else if (targetHandler.startPoint.y === targetHandler.endPoint.y) {
							//xMove
							targetHandler.handler = (function (startPoint, endPoint) {
								var x1 = (startPoint.x > endPoint.x) ? endPoint.x : startPoint.x;
								var x2 = startPoint.x + endPoint.x - x1;
								return function () {
									if (this.direction.indexOf('left') > -1) {
										if (this.position.x > x1) {
											this.position.x--;
										}
									} else if (this.direction.indexOf('right') > -1) {
										if (this.position.x < x2) {
											this.position.x++;
										}
									}
								}
							})(targetHandler.startPoint, targetHandler.endPoint);
						}
						else {
							throw new Error('未实现的move');
						}
					}
					targetHandler.handler.call(me, targetHandler.startPoint, targetHandler.endPoint, road);
				}
			}
			me.$dom.css({transform: 'translate(' + position.x + 'px,' + position.y + 'px)'});
			//me.$dom.css({left: position.x, top: position.y});
			me.animate();
		});
	}
};

module.exports = CrossMove;