(function() {
	'use strict';

	// load dependencies
	var animationControl = require('./animation-control.js');
	var LineMove = require('./line-move.js');
	var Hammer = require('./hammer.js');
	var CrossMove = require('./cross-move.js');
	var resourceLoader = require('./resource-loader.js');
	var resouces = [
		'images/track.jpg',
		'images/loading.png',
		'images/headpage/LOGO.png',
		'images/headpage/backgrund.jpg',
		'images/headpage/btn.png',
		'images/headpage/clouds.png',
		'images/mianpage/background.jpg',
		'images/mianpage/floating.png',
		'images/mianpage/head-img.png',
		'images/mianpage/floatWindow/1.png',
		'images/mianpage/floatWindow/2.png',
		'images/mianpage/floatWindow/3.png',
		'images/mianpage/floatWindow/4.png',
		'images/mianpage/floatWindow/5.png',
		'images/mianpage/floatWindow/9.png',
		'images/mianpage/floatWindow/10.png',
		'images/mianpage/floatWindow/8.png',
		'images/mianpage/floatWindow/btn.png',
		'images/mianpage/floatWindow/btn1.png',
		'images/mianpage/floatWindow/btn2.png',
		'images/mianpage/floatWindow/CDK3.png',
		'images/mianpage/floatWindow/dropOut.png',
		'images/mianpage/floatWindow/foodcoupons.png',
	];
	resourceLoader.loadImages(resouces, function(over, count, length, map, percent, fakePercent) {
		if(over) {
			$(".progress").html('100%');
			setTimeout(function () {
				$('.loading-overlay').slideUp();
			}, 1000);
		} else {
			$(".progress").html(percent + '%');
		}
	}, 5000, 50, 'http://static.leanote.top/ttxd/');
	$(document).ready(function() {
		var bgMusic = $('audio').get(0);
		var $btnMusic = $('.btn-music');
		var $upArrow = $('.up-arrow');

		// background music control
		$btnMusic.click(function() {
			if (bgMusic.paused) {
				bgMusic.play();
				$(this).removeClass('paused');
			} else {
				bgMusic.pause();
				$(this).addClass('paused');
			}
		});

		// init Swiper
		new Swiper('.swiper-container', {
			// mousewheelControl: true,
			effect: 'coverflow', // slide, fade, coverflow or flip
			speed: 400,
			direction: 'vertical',
			//initialSlide: 2,
			mousewheelControl: true,
			fade: {
				crossFade: false
			},
			coverflow: {
				rotate: 100,
				stretch: 0,
				depth: 300,
				modifier: 1,
				slideShadows: false // do disable shadows for better performance
			},
			flip: {
				limitRotation: true,
				slideShadows: false // do disable shadows for better performance
			},
			onInit: function(swiper) {
				animationControl.initAnimationItems(); // get items ready for animations
				animationControl.playAnimation(swiper); // play animations of the first slide
			},
			onTransitionStart: function(swiper) { // on the last slide, hide .btn-swipe
				if (swiper.activeIndex === swiper.slides.length - 1) {
					$upArrow.hide();
				} else {
					$upArrow.show();
				}
			},
			onTransitionEnd: function(swiper) { // play animations of the current slide
				animationControl.playAnimation(swiper);
			},
			onTouchStart: function(swiper, event) { // mobile devices don't allow audios to play automatically, it has to be triggered by a user event(click / touch).
				if (!$btnMusic.hasClass('paused') && bgMusic.paused) {
					//  bgMusic.play();
				}
			}
		});

		var pointMap = {
			'fromCss': true
		};
		var pointChain = 'point1 point4 point5 point7 point8-l point8-h point9-h point9-l point9 point10-l point10-h point11 point12 point13 point13-l point13-h point14-h point14-l point15 point17 point19 point20-l point20-h point21-h point21-l point22 point23 point25 point25-l point25-h point26-h point26-l point29 point30';
		var role = new LineMove('#target', pointChain, pointMap, 'point1', 3);
		$('.point').on('click', function() {
			var point = 'point' + $(this).html();
			role.moveTo(point);
		});
		$('.slide-2 .zoom-container').css({
			zoom: $(document).width() / 640
		});
		$('.slide-3 .zoom-container').css({
			zoom: $(document).width() / 288
		});

		var mapControl = {
			init: function() {
				if (this.inited) {
					return;
				}

				var pointMap = {
					fromCss: true,
					// 点的处理可以抽取公用的逻辑
					p1: {
						cross: [{
							'direction': 'down',
							'point': 'p2'
						}]
					},
					p2: {
						cross: [{
							'direction': 'left',
							'point': 'p3'
						}, {
							'direction': 'down',
							'point': 'p13'
						}, {
							'direction': 'up',
							'point': 'p1'
						}]
					},
					p3: {
						cross: [{
							'direction': 'right',
							'point': 'p2'
						}, {
							'direction': 'left||down',
							'point': 'p4'
						}]
					},
					p4: {
						cross: [{
							'direction': 'left||down',
							'point': 'p5',
							'turn': 'down'
						}, {
							'direction': 'right||up',
							'point': 'p3',
							'turn': 'right'
						}]
					},
					p5: {
						cross: [{
							'direction': 'down',
							'point': 'p6',
							'turn': 'down'
						}, {
							'direction': 'right||up',
							'point': 'p4'
						}]
					},
					p6: {
						cross: [{
							'direction': 'up||left',
							'point': 'p5',
							'turn': 'up'
						}, {
							'direction': 'right||down',
							'point': 'p7'
						}]
					},
					p7: {
						cross: [{
							'direction': 'left||up',
							'point': 'p6'
						}, {
							'direction': 'right||down',
							'point': 'p8'
						}]
					},
					p8: {
						cross: [{
							'direction': 'right',
							'point': 'p9'
						}, {
							'direction': 'left||up',
							'point': 'p7'
						}]
					},
					p9: {
						cross: [{
							'direction': 'right||up',
							'point': 'p10'
						}, {
							'direction': 'left',
							'point': 'p8'
						}]
					},
					p10: {
						cross: [{
							'direction': 'right||up',
							'point': 'p11'
						}, {
							'direction': 'left||down',
							'point': 'p9'
						}]
					},
					p11: {
						cross: [{
							'direction': 'up',
							'point': 'p12'
						}, {
							'direction': 'left||down',
							'point': 'p10'
						}]
					},
					p12: {
						cross: [{
							'direction': 'down',
							'point': 'p11'
						}]
					},
					p13: {
						cross: [{
							'direction': 'right',
							'point': 'p18'
						}, {
							'direction': 'up',
							'point': 'p2'
						}, {
							'direction': 'down',
							'point': 'p14'
						}]
					},
					p14: {
						cross: [{
							'direction': 'right||up',
							'point': 'p13',
							'turn': 'up'
						}, {
							'direction': 'left||down',
							'point': 'p15'
						}]
					},
					p15: {
						cross: [{
							'direction': 'right',
							'point': 'p14',
							'turn': 'right up'
						}, {
							'direction': 'left',
							'point': 'p16',
							'turn': 'left up'
						}]
					},
					p16: {
						cross: [{
							'direction': 'left||up',
							'point': 'p17',
							'turn': 'up'
						}, {
							'direction': 'right||down',
							'point': 'p15'
						}]
					},
					p17: {
						cross: [{
							'direction': 'down',
							'point': 'p16'
						}]
					},
					p18: {
						handler: function(prePoint, direction) {
							if (prePoint === 'p19') {
								if (direction.indexOf('left') > -1) {
									this.direction = 'left up';
									this.nextPoint('p13');
								} else if (d.indexOf('right') > -1) {
									this.nextPoint('p19');
								}
							} else {
								if (direction.indexOf('right') > -1 || direction.indexOf('down')) {
									this.nextPoint('p19');
								} else if (d.indexOf('left') > -1 || d.indexOf('up') > -1) {
									this.nextPoint('p13');
									// todo wan18_19
								}
							}
						}
					},
					p19: {
						cross: [{
							'direction': 'right',
							'point': 'p20'
						}, {
							'direction': 'left||up',
							'point': 'p18'
						}]
					},
					p20: {
						cross: [{
							'direction': 'right||down',
							'point': 'p21'
						}, {
							'direction': 'left',
							'point': 'p19'
						}]
					},
					p21: {
						cross: [{
							'direction': 'down',
							'point': 'p22'
						}, {
							'direction': 'left||up',
							'point': 'p20'
						}]
					},
					p22: {
						cross: [{
							'direction': 'up',
							'point': 'p21'
						}, {
							'direction': 'down',
							'point': 'p23'
						}]
					},
					p23: {
						cross: [{
							'direction': 'up',
							'point': 'p22'
						}, {
							'direction': 'left||down',
							'point': 'p24'
						}]
					},
					p24: {
						cross: [{
							'direction': 'up',
							'point': 'p23'
						}, {
							'direction': 'left',
							'point': 'p25'
						}]
					},
					p25: {
						cross: [{
							'direction': 'up||right',
							'point': 'p24'
						}, {
							'direction': 'left',
							'point': 'p26'
						}]
					},
					p26: {
						cross: [{
							'direction': 'right',
							'point': 'p25'
						}, {
							'direction': 'left||up',
							'point': 'p27'
						}]
					},
					p27: {
						cross: [{
							'direction': 'up',
							'point': 'p28'
						}, {
							'direction': 'right||down',
							'point': 'p26'
						}]
					},
					p28: {
						cross: [{
							'direction': 'up',
							'point': 'p29'
						}, {
							'direction': 'right||down',
							'point': 'p27'
						}]
					},
					p29: {
						cross: [{
							'direction': 'down',
							'point': 'p28'
						}, {
							'direction': 'right||up',
							'point': 'p30'
						}]
					},
					p30: {
						cross: [{
							'direction': 'down',
							'point': 'p29'
						}, {
							'direction': 'right||up',
							'point': 'p31'
						}]
					},
					p31: {
						cross: [{
							'direction': 'right',
							'point': 'p32'
						}, {
							'direction': 'left||down',
							'point': 'p30'
						}]
					},
					p32: {
						cross: [{
							'direction': 'left',
							'point': 'p31'
						}]
					}
				};
				var roadHandler = {
					'p1 p2': {},
					'p2 p3': {},
					'p3 p4': {
						paths: [{
							x: 40,
							y: 65
						}, {
							x: 38,
							y: 66
						}, {
							x: 36,
							y: 67
						}, {
							x: 34,
							y: 68
						}, {
							x: 32,
							y: 69
						}, {
							x: 30,
							y: 70
						}, {
							x: 28,
							y: 71
						}, {
							x: 26,
							y: 72
						}],
						handler: wanHandler
					},
					'p4 p5': {
						paths: [{
							x: 26,
							y: 72
						}, {
							x: 25,
							y: 74
						}, {
							x: 24,
							y: 76
						}, {
							x: 23,
							y: 78
						}, {
							x: 22,
							y: 80
						}, {
							x: 21,
							y: 82
						}, {
							x: 20,
							y: 84
						}, {
							x: 19,
							y: 85
						}],
						handler: wanHandler
					},
					'p5 p6': {},
					'p6 p7': {
						paths: [{
							x: 19,
							y: 300
						}, {
							x: 20,
							y: 302
						}, {
							x: 21,
							y: 304
						}, {
							x: 22,
							y: 306
						}, {
							x: 23,
							y: 308
						}, {
							x: 24,
							y: 310
						}, {
							x: 25,
							y: 312
						}, {
							x: 26,
							y: 315
						}],
						handler: wanHandler
					},
					'p7 p8': {
						paths: [{
							x: 26,
							y: 315
						}, {
							x: 28,
							y: 316
						}, {
							x: 30,
							y: 317
						}, {
							x: 32,
							y: 318
						}, {
							x: 34,
							y: 319
						}, {
							x: 36,
							y: 320
						}, {
							x: 38,
							y: 321
						}, {
							x: 40,
							y: 322
						}],
						handler: wanHandler
					},
					'p8 p9': {},
					'p9 p10': {
						paths: [{
							x: 176,
							y: 322
						}, {
							x: 177,
							y: 321
						}, {
							x: 179,
							y: 320
						}, {
							x: 181,
							y: 319
						}, {
							x: 182,
							y: 318
						}, {
							x: 184,
							y: 317
						}, {
							x: 185,
							y: 316
						}, {
							x: 187,
							y: 315
						}],
						handler: wanHandler
					},
					'p10 p11': {
						paths: [{
							x: 187,
							y: 315
						}, {
							x: 188,
							y: 313
						}, {
							x: 189,
							y: 311
						}, {
							x: 191,
							y: 309
						}, {
							x: 192,
							y: 307
						}, {
							x: 193,
							y: 305
						}, {
							x: 194,
							y: 303
						}],
						handler: wanHandler
					},
					'p11 p12': {},
					'p2 p13': {},
					'p13 p14': {},
					'p14 p15': {
						paths: [{
							x: 131,
							y: 250
						}, {
							x: 129,
							y: 253
						}, {
							x: 127,
							y: 256
						}, {
							x: 125,
							y: 259
						}, {
							x: 123,
							y: 262
						}, {
							x: 121,
							y: 265
						}, {
							x: 118,
							y: 267.5
						}, {
							x: 116,
							y: 268.5
						}, {
							x: 114,
							y: 269
						}, {
							x: 112,
							y: 269.5
						}, {
							x: 110,
							y: 270
						}, {
							x: 108,
							y: 270.5
						}, {
							x: 106,
							y: 271
						}],
						handler: wanHandler
					},
					'p15 p16': {
						paths: [{
							x: 106,
							y: 271
						}, {
							x: 104,
							y: 270.5
						}, {
							x: 101,
							y: 270
						}, {
							x: 98,
							y: 269.5
						}, {
							x: 95,
							y: 269
						}, {
							x: 92,
							y: 268
						}, {
							x: 90,
							y: 267
						}, {
							x: 89,
							y: 265
						}, {
							x: 86,
							y: 262
						}, {
							x: 84,
							y: 259
						}, {
							x: 82,
							y: 256
						}, {
							x: 80,
							y: 253
						}, {
							x: 78,
							y: 250
						}],
						handler: wanHandler
					},
					'p16 p17': {},
					'p13 p18': {
						paths: [{
							x: 131,
							y: 96
						}, {
							x: 133,
							y: 98
						}, {
							x: 134,
							y: 99
						}, {
							x: 135,
							y: 100
						}, {
							x: 136,
							y: 101
						}, {
							x: 137,
							y: 102
						}, {
							x: 138,
							y: 103
						}, {
							x: 139,
							y: 104
						}, {
							x: 140,
							y: 105
						}, {
							x: 141,
							y: 106
						}, {
							x: 142,
							y: 107
						}, {
							x: 143,
							y: 108
						}, {
							x: 144,
							y: 110
						}],
						handler: wanHandler
					},
					'p18 p19': {
						paths: [{
							x: 144,
							y: 110
						}, {
							x: 146,
							y: 111
						}, {
							x: 148,
							y: 112
						}, {
							x: 150,
							y: 113
						}, {
							x: 152,
							y: 114
						}, {
							x: 154,
							y: 115
						}, {
							x: 156,
							y: 116
						}, {
							x: 158,
							y: 117
						}, {
							x: 160,
							y: 118
						}],
						handler: wanHandler
					},
					'p19 p20': {},
					'p20 p21': {
						paths: [{
							x: 240,
							y: 118
						}, {
							x: 241,
							y: 119
						}, {
							x: 242,
							y: 120
						}, {
							x: 244,
							y: 121
						}, {
							x: 246,
							y: 122
						}, {
							x: 248,
							y: 123
						}, {
							x: 249,
							y: 124
						}, {
							x: 250,
							y: 125
						}],
						handler: wanHandler
					},
					'p21 p22': {
						paths: [{
							x: 250,
							y: 125
						}, {
							x: 250.5,
							y: 126
						}, {
							x: 251,
							y: 128
						}, {
							x: 251.5,
							y: 130
						}, {
							x: 252,
							y: 132
						}, {
							x: 252.5,
							y: 134
						}, {
							x: 253,
							y: 135
						}],
						handler: wanHandler
					},
					'p22 p23': {},
					'p23 p24': {
						paths: [{
							x: 253,
							y: 400
						}, {
							x: 252,
							y: 401
						}, {
							x: 251.5,
							y: 403
						}, {
							x: 250,
							y: 405
						}, {
							x: 249,
							y: 407
						}, {
							x: 248,
							y: 408
						}],
						handler: wanHandler
					},
					'p24 p25': {
						paths: [{
							x: 248,
							y: 408
						}, {
							x: 247,
							y: 409
						}, {
							x: 245,
							y: 410
						}, {
							x: 243,
							y: 411
						}, {
							x: 241,
							y: 412
						}, {
							x: 240,
							y: 413
						}],
						handler: wanHandler
					},
					'p25 p26': {},
					'p26 p27': {
						paths: [{
							x: 40,
							y: 413
						}, {
							x: 38,
							y: 412.5
						}, {
							x: 36,
							y: 412
						}, {
							x: 33,
							y: 411.5
						}, {
							x: 30,
							y: 411
						}, {
							x: 28,
							y: 410.5
						}, {
							x: 26,
							y: 410
						}],
						handler: wanHandler
					},
					'p27 p28': {
						paths: [{
							x: 26,
							y: 410
						}, {
							x: 25,
							y: 409
						}, {
							x: 24,
							y: 408
						}, {
							x: 23,
							y: 406
						}, {
							x: 22,
							y: 404
						}, {
							x: 21,
							y: 402
						}, {
							x: 20,
							y: 401
						}, {
							x: 19,
							y: 400
						}],
						handler: wanHandler
					},
					'p28 p29': {},
					'p29 p30': {
						paths: [{
							x: 19,
							y: 376
						}, {
							x: 20,
							y: 374
						}, {
							x: 21,
							y: 372
						}, {
							x: 22,
							y: 369
						}, {
							x: 23,
							y: 366
						}, {
							x: 24,
							y: 364
						}],
						handler: wanHandler
					},
					'p30 p31': {
						paths: [{
							x: 24,
							y: 364
						}, {
							x: 26,
							y: 363.5
						}, {
							x: 29,
							y: 363
						}, {
							x: 32,
							y: 362.5
						}, {
							x: 35,
							y: 362
						}, {
							x: 38,
							y: 361.5
						}, {
							x: 40,
							y: 361
						}],
						handler: wanHandler
					},
					'p31 p32': {}
				};
				function wanHandler(startPoint, endPoint, road, position, direction) {
					this.position.x = position.x;
					this.position.y = position.y;
					if (direction == '+') {
						if ('(p3 p4)(p6 p7)(p9 p10)(p14 p15)(p20 p21)(p23 p24)(p26 p27)(p29 p30)'.indexOf('(' + road + ')') > -1) {
							this.direction = '';
						}
					} else {
						if ('(p4 p5)(p7 p8)(p10 p11)(p15 p16)(p21 p22)(p24 p25)(p27 p28)(p30 p31)'.indexOf('(' + road + ')') > -1) {
							this.direction = '';
						}
					}
				};
				this.mapDom = $('.slide-map');
				this.ruleBtn = this.mapDom.find('.item-btn1');
				this.ball = this.mapDom.find('.item-tip1');
				this.ruleImg = this.mapDom.find('.item-tip2');
				this.ruleCloseBtn = this.ruleImg.find('.item-close');
				// this.ballObj = new Ball(this.ball, 'down');
				this.ballObj = new CrossMove('.item-tip1', 'p1', pointMap, 'down', roadHandler);
				var me = this;
				/*this.ballObj.onPoint(function (point) {
				 console.log(point);
				 //me.checkBallPos(point);
				 });*/
				this.ballObj.start();
				this.oxImg = this.mapDom.find('.item-tip3');
				this.oxYesBtn = this.oxImg.find('.item-yes');
				this.oxNoBtn = this.oxImg.find('.item-no');

				this.monkeyImg = this.mapDom.find('.item-tip4');
				this.monkeyYesBtn = this.monkeyImg.find('.item-yes');
				this.monkeyNoBtn = this.monkeyImg.find('.item-no');

				this.dragonImg = this.mapDom.find('.item-tip5');
				this.dragonYesBtn = this.dragonImg.find('.item-yes');
				this.dragonNoBtn = this.dragonImg.find('.item-no');

				this.maskDom = this.mapDom.find('.item-mask');

				this.dragInitPos = [131, 35];
				this.ballPos = [131, 35];
				this.initDrag();

				this.inited = true;
				this.masked = false;
			},

			initDrag: function() {
				var me = this;

				me.positionEmitInter = 100; // 计算位移时间间隔
				me.hammertime = new Hammer(me.mapDom[0], {
					drag_max_touches: 0
				});

				me.hammertime.on("touch", function(ev) {
					me.catchEvt(ev, true);
					var target = ev.target || window.event.srcElement;

					console.log('dragstart');
				});

				me.hammertime.on("touchmove", function(ev) {
					if (!me.holding) {
						return;
					}
					me.catchEvt(ev);
				});

				me.hammertime.on("touchend", function(ev) {
					me.holding = false;
					if (!ev.gesture) {
						return;
					}

					ev.gesture.preventDefault();

					console.log('touchend');
					// me.resetPos();
				});
			},

			catchEvt: function(ev, isTouch) {
				var me = this;
				// console.log(ev.gesture);

				if (me.masked) {
					return;
				}
				if (!isTouch) {
					if (ev.changedTouches[0]) {
						me.dragMove(ev.changedTouches[0].clientX, ev.changedTouches[0].clientY);
					}
					return;
				}
				var touches = ev.gesture.touches;

				if (ev.target.className.indexOf('btn') == -1) {
					ev.gesture.preventDefault();
				}

				for (var t = 0, len = touches.length; t < len; t++) {
					var touchEvt = touches[t];
					var target = touchEvt.target;

					var proxyX = touchEvt.pageX;
					var proxyY = touchEvt.pageY;
					if (isTouch) {
						me.initTouchX = proxyX;
						me.initTouchY = proxyY;

						if (ev.target.className.indexOf('btn') == -1) {
							me.holding = true;
						}
						me.tickTack();

						me.ballPos = [me.dragInitPos[0], me.dragInitPos[1]];
					}
					me.dragMove(proxyX, proxyY);
				}
			},

			dragMove: function(toLeft, toTop) {
				var me = this;
				var moLeft = toLeft - me.initTouchX;
				var moTop = toTop - me.initTouchY;
				//console.log('target:', ' | move:', moLeft, moTop);
				this.ballObj.setDirection(moLeft, moTop);
				if (me.checkTimer) {
					clearTimeout(me.checkTimer);
				}

				if (!me.holding || me.masked) {
					return;
				}
			},

			tickTack: function() {
				console.log('tick tack');
			},

			resetPos: function() {
				console.log('reset post');

				this.holding = false;
				clearTimeout(this.positionEmitter);
				// me.tickTack();
			}

		};
		mapControl.init();

	});
})();