(function () {
    'use strict';

    // load dependencies
    var animationControl = require('./animation-control.js');
    var mixtool = require('mix-h5');

    // tracking
    var tracking = mixtool.tracking;
    // init tracking
    tracking.add('baidu', '2cb083278ff2a669421fccda5759413c'); // baidu site id
    tracking.add('cnzz', '1260089393'); // cnzz site id


    setTimeout(function() {
        tracking.log('loaded');
    },1000);


    $(document).ready(function () {
        var bgMusic = $('audio').get(0);
        var $btnMusic = $('.btn-music');
        var $upArrow = $('.up-arrow');

        // background music control
        $btnMusic.click(function () {
            if (bgMusic.paused) {
                bgMusic.play();
                $(this).removeClass('paused');
            } else {
                bgMusic.pause();
                $(this).addClass('paused');
            }
        });

        // init Swiper
        new Swiper('.swiper-container', {
            mousewheelControl: true,
            effect: 'fade',    // slide, fade, coverflow or flip
            speed: 400,
            direction: 'vertical',
            fade: {
                crossFade: false
            },
            coverflow: {
                rotate: 100,
                stretch: 0,
                depth: 300,
                modifier: 1,
                slideShadows: false     // do disable shadows for better performance
            },
            flip: {
                limitRotation: true,
                slideShadows: false     // do disable shadows for better performance
            },
            onInit: function (swiper) {
                animationControl.initAnimationItems();  // get items ready for animations
                animationControl.playAnimation(swiper); // play animations of the first slide
            },
            onTransitionStart: function (swiper) {     // on the last slide, hide .btn-swipe
                if (swiper.activeIndex === swiper.slides.length - 1) {
                    $upArrow.hide();
                } else {
                    $upArrow.show();
                }
            },
            onTransitionEnd: function (swiper) {       // play animations of the current slide
                animationControl.playAnimation(swiper);
            },
            onTouchStart: function (swiper, event) {    // mobile devices don't allow audios to play automatically, it has to be triggered by a user event(click / touch).
                if (!$btnMusic.hasClass('paused') && bgMusic.paused) {
                    bgMusic.play();
                }
            }
        });

        // hide loading animation since everything is ready
        $('.loading-overlay').slideUp();
    });


    // getUserInfo
    var serverUserInfo = {
        getUrl: 'http://jishub.com/jjrs/lifeinfo',
        setUrl: 'http://jishub.com/jjrs/setlifeinfo',
        init: function () {
            this.openId = window['openId'];

            // test
            this.openId = 'testOpenId0809';

            if (!this.openId) {
                alert('未获取openId');
            }

            this.infoWrap = $('.userinfo-box');
            this.infoTip = this.infoWrap.find('.info');
            this.lostLifeBtn = this.infoWrap.find('.info-minus');
            this.plusLifeBtn = this.infoWrap.find('.info-plus');
            this.shareBtn = this.infoWrap.find('.info-share');
            this.getInfo(this.openId);

            this.bindEvents();
        },

        getInfo: function () {
            var me = this;

            $.ajax({
                type: "GET",
                url: me.getUrl,
                data: {
                    openId: me.openId
                },
                dataType: "json",
                success: function (info) {
                    console.log(info);
                    if (info && info.lifeinfo) {
                        me.userInfo = info.lifeinfo;
                        me.setInfo();
                    }
                    else {
                        console.log('获取用户信息失败。');
                    }
                },
                error: function (msg) {
                    alert(msg);
                }
            });
        },

        setInfo: function () {
            var infoStr = 'lostLife:' + this.userInfo.lostLife
                + ' | shareFlag:' + this.userInfo.shareFlag;

            infoStr += ' | openId:' + this.openId;
            this.infoTip.html(infoStr);
        },

        munisLife: function (isAdd) {
            var me = this;

            if (me.userInfo && (me.userInfo.lostLife < 3 || isAdd || (me.userInfo.shareFlag == 1 && me.userInfo.lostLife < 4))) {
                var toLife;
                if (isAdd) {
                    if (me.userInfo.lostLife * 1 !== 0) {
                        toLife = me.userInfo.lostLife * 1 - 1;
                    }
                    else {
                        alert('当前满血');
                        return;
                    }
                }
                else {
                    toLife = me.userInfo.lostLife * 1 + 1;
                }
                $.ajax({
                    type: "GET",
                    url: me.setUrl,
                    data: {
                        openId: me.openId,
                        lostLife: toLife
                    },
                    dataType: "json",
                    success: function (info) {
                        console.log(info);
                        if (info && info.lifeinfo) {
                            me.userInfo = info.lifeinfo;
                            me.setInfo();
                        }
                        else {
                            console.log('设置用户信息失败。');
                        }
                    },
                    error: function (msg) {
                        alert(msg);
                    }
                });
            }
            else {
                alert('已无命');
            }
        },

        bindEvents: function () {
            var me = this;
            me.lostLifeBtn.click(function () {
                me.munisLife();
                tracking.log('munisLife');
            });

            me.plusLifeBtn.click(function () {
                me.munisLife(true);
                tracking.log('plusLife');
            });

            me.shareBtn.click(function () {
                $.ajax({
                    type: "GET",
                    url: me.setUrl,
                    data: {
                        openId: me.openId,
                        shareFlag: 1
                    },
                    dataType: "json",
                    success: function (info) {
                        console.log(info);
                        if (info && info.lifeinfo) {
                            me.userInfo = info.lifeinfo;
                            me.setInfo();
                        }
                        else {
                            console.log('设置用户信息失败。');
                        }
                    },
                    error: function (msg) {
                        alert(msg);
                    }
                });
            });
        }
    };

    serverUserInfo.init();
})();
